;*******************************************************************************
;*
;*      Titel:  Unterprogramme zur Nutzung des CO2 Sensors K30 von SenseAir
;*              ueber die I2C-Schnittstelle (TWI) (SR_CO2.asm)
;*
;*      Datum:  01/04/10        Version:        0.2 (23/11/2012)
;*      Autor:  WEIGU
;*
;*      Informationen zur Beschaltung:
;*      Prozessor:      ATmega32        Quarzfrequenz:  16MHz
;*      Eingaenge:      SDA
;*      Ausgaenge:      SCL
;*
;*      Informationen zur Funktionsweise:
;*
;*      Im Sensor (G+ = 5V) werden ueber einen DC/DC Konverter 3,3V an DVCC
;*      bereitgestellt. 2 Pull-Up-Widerstaende von 1,2k ziehen SCL und SDA
;*      nach 3,3V.
;*      r24 und r25 dienen der Parameteruebergabe CO2 Werte im Ram an der
;*      symbolischen Adresse CO2+1 (HighByte) und CO2+2 (LowByte)
;*
;*      Werte in RAM-Tabelle (symbolischer Name CO2, Im HP reservieren)
;*      CO2 LowByte in CO2+2 und in r24
;*      CO2 HighByte in CO2+1 und in r25
;*
;*      Copyright (c) 2012   Guy WEILER         weigu[at]weigu[dot]lu
;*
;*      Hiermit wird unentgeltlich, jeder Person, die eine Kopie der Software
;*      und der zugehoerigen Dokumentationen (die "Software") erhaelt, die
;*      Erlaubnis erteilt, uneingeschraenkt zu benutzen, inklusive und ohne
;*      Ausnahme, dem Recht, sie zu verwenden, kopieren, aendern, fusionieren,
;*      verlegen, verbreiten, unterlizenzieren und/oder zu verkaufen, und
;*      Personen, die diese Software erhalten, diese Rechte zu geben, unter den
;*      folgenden Bedingungen:
;*
;*      ;Der obige Urheberrechtsvermerk und dieser Erlaubnisvermerk sind in alle
;*      ;Kopien oder Teilkopien der Software beizulegen.
;*      ;
;*      ;DIE SOFTWARE WIRD OHNE JEDE AUSDRUECKLICHE ODER IMPLIZIERTE GARANTIE
;*      ;BEREITGESTELLT, EINSCHLIESSLICH DER GARANTIE ZUR BENUTZUNG FUER DEN
;*      ;VORGESEHENEN ODER EINEM BESTIMMTEN ZWECK SOWIE JEGLICHER RECHTS-
;*      ;VERLETZUNG, JEDOCH NICHT DARAUF BESCHRAENKT. IN KEINEM FALL SIND DIE
;*      ;AUTOREN ODER COPYRIGHTINHABER FUER JEGLICHEN SCHADEN ODER SONSTIGE
;*      ;ANSPRUECHE HAFTBAR ZU MACHEN, OB INFOLGE DER ERFUELLUNG EINES
;*      ;VERTRAGES, EINES DELIKTES ODER ANDERS IM ZUSAMMENHANG MIT DER SOFTWARE
;*      ;ODER SONSTIGER VERWENDUNG DER SOFTWARE ENTSTANDEN.
;*
;*******************************************************************************
;Damit der Code im Flash gering bleibt werden ausschliesslich die benutzten
;Unterprogramme freigeschaltet (Flag = 1)
.EQU    Flag_CO2RD  = 1		;CO2 Werte abholen (default = 1)
.EQU    Flag_CO2REP = 0		;Lese Sensoradresse aus dem EEPROM (r24)
.EQU    Flag_CO2SEP = 0		;Schreibe neue Sensoradresse (r24) ins EEPROM

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;       Reservierungen und Zuweisungen
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.DSEG
CO2:	.BYTE 9         ;xxxxppm + Leerzeichen + Nullbyte (ASCII)
.CSEG

;-------------------------------------------------------------------------------
;       CO2 Wert vom Sensor abholen
;-------------------------------------------------------------------------------
.IF Flag_CO2RD == 1
CO2RD:  push    r16
	push    r17
        push    r18
        push    r24

        clr	r17		;Fehlerzaehler
        ldi     r18,3		;Zaehler fuer Fehlversuche
        ;Wake up CO2 Sensor (sende Nullbyte ueber I2C)
CO2RDW: rcall   I2CSTA          ;Start
        ldi     r24,0x00
        rcall   I2CSND
        rcall   I2CSTO          ;Stop
        ;Warte 2 ms bis der CO2 Sensor bereit ist
        rcall   W5ms

        ;CO2-Wert anfragen (S29 I2C Datenblatt des Sensors)
CO2RD1: rcall   I2CSTA          ;Start, 1 Rahmen, Initialisierung
        cpi     r25,0x08        ;Start erfolgreich? sonst Fehler
        brne    CO2RDE          ;
        ldi     r24,0x22        ;Sensoradresse (0x11) + R/W = 0 (2^0,Schreiben)
        rcall   I2CSND		;= 0x22
        cpi     r25,0x18        ;Slave Adresse erfolgreich sonst Fehler
        brne    CO2RDE
        ldi     r24,0x22        ;Kommando 2 (ReadRAM) 2 Bytes schreiben
        rcall   I2CSND
        cpi     r25,0x28        ;Datenbyte erfolgreich? sonst Fehler
        brne    CO2RDE          ;
        ldi     r24,0x00        ;RAM Adresse schreiben (2 Byte) CO2 auf 0x0008
        rcall   I2CSND
        cpi     r25,0x28        ;Datenbyte erfolgreich? sonst Fehler
        brne    CO2RDE          ;
        ldi     r24,0x08
        rcall   I2CSND
        cpi     r25,0x28        ;Datenbyte erfolgreich? sonst Fehler
        brne    CO2RDE          ;
        ldi     r24,0x2A        ;Checksumme (0xA2) senden
        rcall   I2CSND
        cpi     r25,0x28        ;Datenbyte erfolgreich? sonst Fehler
        brne    CO2RDE          ;
        rcall   I2CSTO          ;Stop
        
        rcall   W10ms           ;Wait 20 ms for Processing Request on Sensor
        rcall   W10ms
        
        rcall   I2CSTA          ;Start, 2 Rahmen, Lesen des CO2 Wertes
        cpi     r25,0x08        ;Start erfolgreich? sonst Fehler
        brne    CO2RDE        
        ldi     r24,0x23        ;Sensoradresse (0x11) + R/W = 1 (2^0,Lesen)
        rcall   I2CSND		;= 0x23
        cpi     r25,0x40        ;Slave Adresse erfolgreich sonst Fehler
        brne    CO2RDE
        rcall   I2CRAK          ;Daten empfangen mit Bestaetigung (ACK)
        cpi     r25,0x50        ;Datenbyte empfangen mit ACK
        brne    CO2RDE
        sts     CO2+0,r24       ;Sensor-Statusbyte nach CO2
        rcall   I2CRAK
        cpi     r25,0x50        ;Datenbyte empfangen mit ACK
        brne    CO2RDE
        sts     CO2+1,r24       ;Wert HighByte CO2 nach CO2+1
        rcall   I2CRAK
        cpi     r25,0x50        ;Datenbyte empfangen mit ACK
        brne    CO2RDE
        sts     CO2+2,r24       ;Wert LowByte CO2 nach CO2+2
        rcall   I2CRNA          ;Daten empfangen ohne Bestaetigung (NACK)
        cpi     r25,0x58        ;Datenbyte empfangen ohne ACK
        brne    CO2RDE
        sts     CO2+3,r24       ;Checksumme nach CO2+3
        rcall   I2CSTO          ;Stop

        lds     r16,CO2+0       ;Ueberpruefe Statusbit
        cpi     r16,0x21
        breq	CO2RD3		;zur Umwandlung
        rjmp    CO2RDF		;Fehler

CO2RDE: rcall   I2CSTO          ;Stop
CO2RDF: inc	r17		;inkrementiere Fehlerflag
	dec     r18
        breq    CO2RDR		;raus
        rcall	W100ms		;Warte 100ms
        rcall	W100ms		;Warte 100ms
        rjmp    CO2RDW		;Sensor neu aktivieren!


        ;Umwandeln von BCD (2 Byte) nach ASCII (4 Byte + Text 'ppm ' + 
        ;Nullbyte = 9 Byte, Tabelle CO2)
CO2RD3: lds     r1,CO2+1
        lds     r0,CO2+2
        ldi     r24,0x30
        ldi     r16,100
        mov     r2,r16
        rcall   DIV21
        mov     r25,r1
        ldi     r16,0
        mov     r1,r16
        ldi     r16,10
        mov     r2,r16
        rcall   DIV21
        add     r0,r24
        sts     CO2,r0
        add     r1,r24
        sts     CO2+1,r1
        mov     r0,r25
        ldi     r16,0
        mov     r1,r16
        ldi     r16,10
        mov     r2,r16
        rcall   DIV21
        add     r0,r24
        sts     CO2+2,r0
        add     r1,r24
        sts     CO2+3,r1
        ldi     r16,'p'         ;Konstanten
        sts     CO2+4,r16
        sts     CO2+5,r16
        ldi     r16,'m'
        sts     CO2+6,r16
        ldi     r16,' '
        sts     CO2+7,r16
        clr     r16
        sts     CO2+8,r16
        

CO2RDR:	mov	r25,r17
	pop     r24
	pop     r18
	pop	r17
        pop     r16
        ret
.ENDIF
;-------------------------------------------------------------------------------
;       Lese Sensoradresse aus dem EEPROM (Adresse in r24)
;-------------------------------------------------------------------------------
.IF Flag_CO2REP == 1
CO2REP: push    r16
        rcall   I2CSTA          ;Start, 1 Rahmen, Initialisierung
        ldi     r24,0xFE        ;Sensoradresse (0x7F) + R/W = 0 (2^0,Schreiben)
        rcall   I2CSND
        ldi     r24,0x41        ;Kommando 4 (ReadEprom) 1 Byte 
        rcall   I2CSND
        ldi     r24,0x00        ;Eprom Adresse schreiben (2 Byte) 
        rcall   I2CSND          ;Sensoradresse auf 0x0000
        ldi     r24,0x00
        rcall   I2CSND
        ldi     r24,0x41        ;Checksumme senden
        rcall   I2CSND
        rcall   I2CSTO          ;Stop
        rcall   W10ms           ;Wait 20 ms for Processing Request on Sensor
        rcall   W10ms
        rcall   I2CSTA          ;Start, 2 Rahmen, Lesen des CO2 Wertes
        ldi     r24,0xFF        ;Sensoradresse (0x7F) + R/W = 1 (2^0,Lesen)
        rcall   I2CSND
        rcall   I2CRAK          ;Daten empfangen mit Bestaetigung (ACK)
        rcall   I2CRAK          ;Daten empfangen mit Bestaetigung (ACK)
        mov     r16,r24         ;
        rcall   I2CRNA          ;Daten empfangen ohne Bestaetigung (NACK)
        rcall   I2CSTO          ;Stop
        mov     r24,r16         ;Sensoradresse in r24
        pop     r16
        ret
.ENDIF
;-------------------------------------------------------------------------------
;       Schreibe neue Sensoradresse (0x11) ins EEPROM
;-------------------------------------------------------------------------------
.IF Flag_CO2SEP == 1
CO2SEP: rcall   I2CSTA          ;Start, 1 Rahmen, Initialisierung
        ldi     r24,0xFE        ;Sensoradresse (0x7F) + R/W = 0 (2^0,Schreiben)
        rcall   I2CSND
        ldi     r24,0x31        ;Kommando 3 (WriteEprom) 1 Byte
        rcall   I2CSND
        ldi     r24,0x00        ;Eprom Adresse schreiben (2 Byte)
        rcall   I2CSND          ;Sensoradresse auf 0x0000
        ldi     r24,0x00
        rcall   I2CSND
        ldi     r24,0x11		;neue Sensoradresse! = 0x11
        rcall   I2CSND
        ldi     r24,0x42        ;Checksumme senden
        rcall   I2CSND
        rcall   I2CSTO          ;Stop
        rcall   W10ms           ;Wait 20 ms for Processing Request on Sensor
        rcall   W10ms
        ret
.ENDIF

