;*******************************************************************************
;*
;*      Titel:  Unterprogramme zur Nutzung des Sensors Temperatur und 
;*              Feuchtigskeitsensors SHT75 von SENSIRION
;*              (SR_SHT75.asm)
;*
;*      Datum:  15/04/10        Version:        0.2 (11/02/13)
;*      Autor:  WEIGU
;*
;*      Informationen zur Beschaltung:
;*      Prozessor:      ATmega32        Quarzfrequenz:  16MHz
;*      Eingaenge:      DATA siehe Zuweisung
;*      Ausgaenge:      SCK  siehe Zuweisung
;*
;*      Informationen zur Funktionsweise:
;*      
;*      Unterprogramme:
;*      SHTRHU:	SHT75 Luftfeuchtigkeit (12 Bit) messen,
;*      		Daten in r25:r24 (dauert 80ms)
;*      		ASCII: 7 Byte xx,xx% + Nullbyte in SRAM Tabelle HUM1
;*      		4 Byte xx% + Nullbyte in SRAM Tabelle HUM2 (gerundet)
;*      	SHTRTE:	SHT75 Temperatur (14 Bit) messen,
;*      		 Daten in r25:r24 (dauert ungef. 320ms)
;*      		 ASCII: 9 Byte xxx,xx°C + Nullbyte in SRAM Tabelle TEMP1
;*      		 5 Byte xx°C + Nullbyte in SRAM Tabelle TEMP2 (gerundet)
;*      
;*      Copyright (c) 2010   Guy WEILER         weigu[at]weigu[dot]lu
;*
;*      Hiermit wird unentgeltlich, jeder Person, die eine Kopie der Software
;*      und der zugehoerigen Dokumentationen (die "Software") erhaelt, die
;*      Erlaubnis erteilt, uneingeschraenkt zu benutzen, inklusive und ohne
;*      Ausnahme, dem Recht, sie zu verwenden, kopieren, aendern, fusionieren,
;*      verlegen, verbreiten, unterlizenzieren und/oder zu verkaufen, und
;*      Personen, die diese Software erhalten, diese Rechte zu geben, unter den
;*      folgenden Bedingungen:
;*
;*      ;Der obige Urheberrechtsvermerk und dieser Erlaubnisvermerk sind in alle
;*      ;Kopien oder Teilkopien der Software beizulegen.
;*      ;
;*      ;DIE SOFTWARE WIRD OHNE JEDE AUSDRUECKLICHE ODER IMPLIZIERTE GARANTIE
;*      ;BEREITGESTELLT, EINSCHLIESSLICH DER GARANTIE ZUR BENUTZUNG FUER DEN
;*      ;VORGESEHENEN ODER EINEM BESTIMMTEN ZWECK SOWIE JEGLICHER RECHTS-
;*      ;VERLETZUNG, JEDOCH NICHT DARAUF BESCHRAENKT. IN KEINEM FALL SIND DIE
;*      ;AUTOREN ODER COPYRIGHTINHABER FUER JEGLICHEN SCHADEN ODER SONSTIGE
;*      ;ANSPRUECHE HAFTBAR ZU MACHEN, OB INFOLGE DER ERFUELLUNG EINES
;*      ;VERTRAGES, EINES DELIKTES ODER ANDERS IM ZUSAMMENHANG MIT DER SOFTWARE
;*      ;ODER SONSTIGER VERWENDUNG DER SOFTWARE ENTSTANDEN.
;*
;*******************************************************************************

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;       Reservierungen und Zuweisungen
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.DSEG
TEMP:           .BYTE 2         ;Rest + Quotient (fuer Temperaturkompensation)
TEMP1:          .BYTE 10        ;xxx,xx°C + Leerzeichen + Nullbyte
TEMP2:          .BYTE 6         ;xx°C + Leerzeichen + Nullbyte
SORH:           .BYTE 2         ;Rest + Quotient (fuer Temperaturkompensation)
TCFLAG:         .BYTE 1         ;Flag fuer Temperaturkompensation
HUM1:           .BYTE 7         ;xx,xx% + Nullbyte
HUM2:           .BYTE 4         ;xx% + Nullbyte
.CSEG

.EQU    SHTSCK = 7
.EQU    SHTDATA = 6
.EQU    SHTPORT = PORTC
.EQU    SHTDDR = DDRC
.EQU    SHTPIN = PINC
        
;-------------------------------------------------------------------------------
;       SHT75 Luftfeuchtigkeit (12 Bit) messen, Daten in r25:r24 (dauert 80ms)
;       ASCII: 7 Byte xx,xx% + Nullbyte in SRAM Tabelle HUM1
;              4 Byte xx% + Nullbyte in SRAM Tabelle HUM2 (gerundet)

;-------------------------------------------------------------------------------
SHTRHU: push    r16
        clr     r24             ;
        clr     r25
        ;Pins initialisieren
        sbi     SHTDDR,SHTSCK   ;SCK = Ausgang
        sbi     SHTDDR,SHTDATA  ;DATA = Ausgang
        sts     TCFLAG,r24      ;Flag fuer Temperaturkompensation loeschen
        ;Transmission Start                DATA           SCK
        sbi     SHTPORT,SHTSCK  ;                        -------
        sbi     SHTPORT,SHTDATA ;        -------                |
        rcall   W1us            ;               |               |
        cbi     SHTPORT,SHTDATA ;        -------                |
        rcall   W1us            ;       |                       |
        cbi     SHTPORT,SHTSCK  ;       |                -------
        rcall   W1us            ;       |               |
        rcall   W1us            ;       |               |
        sbi     SHTPORT,SHTSCK  ;       |                -------
        rcall   W1us            ;       |                       |
        sbi     SHTPORT,SHTDATA ;        -------                |
        rcall   W1us            ;               |               |
        cbi     SHTPORT,SHTSCK  ;               |        -------
        rcall   W1us            ;               |       |
        rcall   W1us            ;               |       |
        cbi     SHTPORT,SHTDATA ;        -------        |
        rcall   W1us            ;       |               |
        rcall   W1us            ;       |               |
        rcall   W1us            ;       |               |
        ;Adress 0b000           ;
        sbi     SHTPORT,SHTSCK  ;       |                -------
        rcall   W1us            ;       |                       | A2
        rcall   W1us            ;       |                       |
        cbi     SHTPORT,SHTSCK  ;       |                -------
        rcall   W1us            ;       |               |
        rcall   W1us            ;       |               |
        sbi     SHTPORT,SHTSCK  ;       |                -------
        rcall   W1us            ;       |                       | A1
        rcall   W1us            ;       |                       |
        cbi     SHTPORT,SHTSCK  ;       |                -------
        rcall   W1us            ;       |               |
        rcall   W1us            ;       |               |
        sbi     SHTPORT,SHTSCK  ;       |                -------
        rcall   W1us            ;       |                       | A0
        rcall   W1us            ;       |                       |
        cbi     SHTPORT,SHTSCK  ;       |                -------
        rcall   W1us            ;       |               |
        rcall   W1us            ;       |               |
        ;Command 00101 Measure Humidity
        sbi     SHTPORT,SHTSCK  ;       |                -------
        rcall   W1us            ;       |                       | C4
        rcall   W1us            ;       |                       |
        cbi     SHTPORT,SHTSCK  ;       |                -------
        rcall   W1us            ;       |               |
        rcall   W1us            ;       |               |
        sbi     SHTPORT,SHTSCK  ;       |                -------
        rcall   W1us            ;       |                       | C3
        rcall   W1us            ;       |                       |
        cbi     SHTPORT,SHTSCK  ;       |                -------
        rcall   W1us            ;       |               |
        sbi     SHTPORT,SHTDATA ;        -------        |
        rcall   W1us            ;               |       |
        sbi     SHTPORT,SHTSCK  ;               |        -------
        rcall   W1us            ;               |               | C2
        rcall   W1us            ;               |               |
        cbi     SHTPORT,SHTSCK  ;               |        -------
        rcall   W1us            ;               |       |
        cbi     SHTPORT,SHTDATA ;        -------        |
        rcall   W1us            ;       |               |
        sbi     SHTPORT,SHTSCK  ;       |                -------
        rcall   W1us            ;       |                       | C1
        rcall   W1us            ;       |                       |
        cbi     SHTPORT,SHTSCK  ;       |                -------
        rcall   W1us            ;       |               |
        sbi     SHTPORT,SHTDATA ;        -------        |
        rcall   W1us            ;               |       |
        sbi     SHTPORT,SHTSCK  ;               |        -------
        rcall   W1us            ;               |               | C0
        rcall   W1us            ;               |               |
        cbi     SHTPORT,SHTSCK  ;               |        -------
        cbi     SHTDDR,SHTDATA  ;DATA = Eingang
        sbi     SHTPORT,SHTDATA ;Pull-Up
        rcall   W1us            ;               |       |
        rcall   W1us            ;               |       |
        sbi     SHTPORT,SHTSCK  ;               |        -------
SHTRH1: sbic    SHTPIN,SHTDATA  ;        -------                | ACK
        rjmp    SHTRH1          ;       | ACK                   |
        cbi     SHTPORT,SHTSCK  ;       |                -------
SHTRH2: sbis    SHTPIN,SHTDATA  ;        -------        |
        rjmp    SHTRH2          ;               |       |
        rcall   W10ms           ;               |       |
SHTRH3: sbic    SHTPIN,SHTDATA  ;        -------        |
        rjmp    SHTRH3          ;       |               |
        rcall   W1us            ;       |               |
        ;Humidity Data
        sbi     SHTPORT,SHTSCK  ;       |                -------
        rcall   W1us            ;       |                       | H15
        rcall   W1us            ;       |                       |
        sbic    SHTPIN,SHTDATA  ;        *******                |
        ori     r25,0x80        ;        *******                |
        cbi     SHTPORT,SHTSCK  ;        *******         -------
        rcall   W1us            ;        *******        |
        rcall   W1us            ;        *******        |
        sbi     SHTPORT,SHTSCK  ;        *******         -------
        rcall   W1us            ;        *******                | H14
        rcall   W1us            ;        *******                |
        sbic    SHTPIN,SHTDATA  ;        *******                |
        ori     r25,0x40        ;        *******                |
        cbi     SHTPORT,SHTSCK  ;        *******         -------
        rcall   W1us            ;        *******        |
        rcall   W1us            ;        *******        |
        sbi     SHTPORT,SHTSCK  ;        *******         -------
        rcall   W1us            ;        *******                | H13
        rcall   W1us            ;        *******                |
        sbic    SHTPIN,SHTDATA  ;        *******                |
        ori     r25,0x20        ;        *******                |
        cbi     SHTPORT,SHTSCK  ;        *******         -------
        rcall   W1us            ;        *******        |
        rcall   W1us            ;        *******        |
        sbi     SHTPORT,SHTSCK  ;        *******         -------
        rcall   W1us            ;        *******                | H12
        rcall   W1us            ;        *******                |
        sbic    SHTPIN,SHTDATA  ;        *******                |
        ori     r25,0x10        ;        *******                |
        cbi     SHTPORT,SHTSCK  ;        *******         -------
        rcall   W1us            ;        *******        |
        rcall   W1us            ;        *******        |
        sbi     SHTPORT,SHTSCK  ;        *******         -------
        rcall   W1us            ;        *******                | H11
        rcall   W1us            ;        *******                |
        sbic    SHTPIN,SHTDATA  ;        *******                |
        ori     r25,0x08        ;        *******                |
        cbi     SHTPORT,SHTSCK  ;        *******         -------
        rcall   W1us            ;        *******        |
        rcall   W1us            ;        *******        |
        sbi     SHTPORT,SHTSCK  ;        *******         -------
        rcall   W1us            ;        *******                | H10
        rcall   W1us            ;        *******                |
        sbic    SHTPIN,SHTDATA  ;        *******                |
        ori     r25,0x04        ;        *******                |
        cbi     SHTPORT,SHTSCK  ;        *******         -------
        rcall   W1us            ;        *******        |
        rcall   W1us            ;        *******        |
        sbi     SHTPORT,SHTSCK  ;        *******         -------
        rcall   W1us            ;        *******                | H9
        rcall   W1us            ;        *******                |
        sbic    SHTPIN,SHTDATA  ;        *******                |
        ori     r25,0x02        ;        *******                |
        cbi     SHTPORT,SHTSCK  ;        *******         -------
        rcall   W1us            ;        *******        |
        rcall   W1us            ;        *******        |
        sbi     SHTPORT,SHTSCK  ;        *******         -------
        rcall   W1us            ;        *******                | H8
        rcall   W1us            ;        *******                |
        sbic    SHTPIN,SHTDATA  ;        *******                |
        ori     r25,0x01        ;        *******                |
        cbi     SHTPORT,SHTSCK  ;        *******         -------
        rcall   W1us            ;        *******        |
        rcall   W1us            ;        *******        |
        rcall   W1us            ;        *******        |
        rcall   W1us            ;        *******        |
        rcall   W1us            ;        *******        |
        sbi     SHTDDR,SHTDATA  ;DATA = Ausgang
        cbi     SHTPORT,SHTDATA;        | ACK           |
        rcall   W1us            ;       |               |
        sbi     SHTPORT,SHTSCK  ;       |                -------
        rcall   W1us            ;       |                       | ACK
        rcall   W1us            ;       |                       |
        cbi     SHTPORT,SHTSCK  ;       |                -------
        rcall   W1us            ;       |               |
        rcall   W1us            ;       |               |
        cbi     SHTDDR,SHTDATA  ;DATA = Eingang
        sbi     SHTPORT,SHTDATA ;Pull-Up
        sbi     SHTPORT,SHTSCK  ;       |                -------
        rcall   W1us            ;       |                       | H7
        rcall   W1us            ;       |                       |
        sbic    SHTPIN,SHTDATA  ;        *******                |
        ori     r24,0x80        ;        *******                |
        cbi     SHTPORT,SHTSCK  ;        *******         -------
        rcall   W1us            ;        *******        |
        rcall   W1us            ;        *******        |
        sbi     SHTPORT,SHTSCK  ;        *******         -------
        rcall   W1us            ;        *******                | H6
        rcall   W1us            ;        *******                |
        sbic    SHTPIN,SHTDATA  ;        *******                |
        ori     r24,0x40        ;        *******                |
        cbi     SHTPORT,SHTSCK  ;        *******         -------
        rcall   W1us            ;        *******        |
        rcall   W1us            ;        *******        |
        sbi     SHTPORT,SHTSCK  ;        *******         -------
        rcall   W1us            ;        *******                | H5
        rcall   W1us            ;        *******                |
        sbic    SHTPIN,SHTDATA  ;        *******                |
        ori     r24,0x20        ;        *******                |
        cbi     SHTPORT,SHTSCK  ;        *******         -------
        rcall   W1us            ;        *******        |
        rcall   W1us            ;        *******        |
        sbi     SHTPORT,SHTSCK  ;        *******         -------
        rcall   W1us            ;        *******                | H4
        rcall   W1us            ;        *******                |
        sbic    SHTPIN,SHTDATA  ;        *******                |
        ori     r24,0x10        ;        *******                |
        cbi     SHTPORT,SHTSCK  ;        *******         -------
        rcall   W1us            ;        *******        |
        rcall   W1us            ;        *******        |
        sbi     SHTPORT,SHTSCK  ;        *******         -------
        rcall   W1us            ;        *******                | H3
        rcall   W1us            ;        *******                |
        sbic    SHTPIN,SHTDATA  ;        *******                |
        ori     r24,0x08        ;        *******                |
        cbi     SHTPORT,SHTSCK  ;        *******         -------
        rcall   W1us            ;        *******        |
        rcall   W1us            ;        *******        |
        sbi     SHTPORT,SHTSCK  ;        *******         -------
        rcall   W1us            ;        *******                | H2
        rcall   W1us            ;        *******                |
        sbic    SHTPIN,SHTDATA  ;        *******                |
        ori     r24,0x04        ;        *******                |
        cbi     SHTPORT,SHTSCK  ;        *******         -------
        rcall   W1us            ;        *******        |
        rcall   W1us            ;        *******        |
        sbi     SHTPORT,SHTSCK  ;        *******         -------
        rcall   W1us            ;        *******                | H1
        rcall   W1us            ;        *******                |
        sbic    SHTPIN,SHTDATA  ;        *******                |
        ori     r24,0x02        ;        *******                |
        cbi     SHTPORT,SHTSCK  ;        *******         -------
        rcall   W1us            ;        *******        |
        rcall   W1us            ;        *******        |
        sbi     SHTPORT,SHTSCK  ;        *******         -------
        rcall   W1us            ;        *******                | H0
        rcall   W1us            ;        *******                |
        sbic    SHTPIN,SHTDATA  ;        *******                |
        ori     r24,0x01        ;        *******                |
        cbi     SHTPORT,SHTSCK  ;        *******         -------
        rcall   W1us            ;        *******        |
        rcall   W1us            ;        *******        |
        rcall   W1us            ;        *******        |
        rcall   W1us            ;        *******        |
        rcall   W1us            ;        *******        |
        sbi     SHTDDR,SHTDATA  ;DATA = Ausgang
        sbi     SHTPORT,SHTDATA;                |       |
        rcall   W1us            ;               |       |
        sbi     SHTPORT,SHTSCK  ;               |        -------
        rcall   W1us            ;               |               | ACK
        rcall   W1us            ;               |               |
        cbi     SHTPORT,SHTSCK  ;               |        -------
        rcall   W1us            ;               |       |
        ;Feuchtigskeitwert linearisieren und nach ASCII
        ;RH = -2,0468 + 0,0367*SORH - 1,5955E6*SORH^2)
        sts     SORH,r24        ;SORH retten fuer Temperaturkompensation
        sts     SORH+1,r25
        mov     r6,r24          ; 1,5955E6*SORH^2
        mov     r7,r25          ;Quadrat bilden
        mov     r8,r24
        mov     r9,r25
        rcall   MUL22
        mov     r8,r2           ;*15955 (0x3E53)
        mov     r9,r3
        mov     r10,r4
        mov     r11,r5
        ldi     r16,LOW(15955)
        mov     r12,r16
        ldi     r16,HIGH(15955)
        mov     r13,r16
        rcall   MUL42
        mov     r0,r2           ;/1000
        mov     r1,r3
        mov     r2,r4
        mov     r3,r5
        mov     r4,r6
        mov     r5,r7
        ldi     r16,LOW(1000)
        mov     r6,r16
        ldi     r16,HIGH(1000)
        mov     r7,r16
        rcall   DIV62
        clr     r4
        clr     r5
        ldi     r16,LOW(1000)   ;/1000
        mov     r6,r16
        ldi     r16,HIGH(1000)
        mov     r7,r16
        rcall   DIV62
        mov     r10,r0          ;gewichtetes Quadrat nach r10:r13
        mov     r11,r1
        mov     r12,r2
        mov     r13,r3
        mov     r6,r24          ;0,0367*SORH
        mov     r7,r25
        ldi     r16,LOW(367)
        mov     r8,r16
        ldi     r16,HIGH(367)
        mov     r9,r16
        rcall   MUL22
        sub     r2,r10
        sbc     r3,r11
        sbc     r4,r12
        sbc     r5,r13
        ldi     r16,LOW(20468)   ;2,0468 (20468 = 0x4FF4)
        sub     r2,r16
        ldi     r16,HIGH(20468)
        sbc     r3,r16
        sbc     r4,Zero
        sbc     r5,Zero
        mov     r0,r2           ;/10000
        mov     r1,r3
        mov     r2,r4
        mov     r3,r5
        ldi     r16,LOW(10000)
        mov     r4,r16
        ldi     r16,HIGH(10000)
        mov     r5,r16
        rcall   DIV42
        mov     r25,r0          ;Quotient RH in r25 (r1 ignorieren)
        mov     r0,r2           ;Rest auf 2 Stellen reduzieren
        mov     r1,r3
        ldi     r16,100
        mov     r2,r16
        rcall   DIV21
        mov     r24,r0          ;Rest RH in r24
        

        ;Temperaturkompensation
        ;RH = (T-25)(0.01+0.00008*SORH)+RH = (T-25)((1000+8)*SORH)/1000/100+RH
        lds     r16,TEMP+1
        subi    r16,25          ;1 Zahl der Mul. (T-25) in r6:r7
        mov     r6,r16
        clr     r7
        sbrs    r6,7
        rjmp    SHTRH4
        com     r6              ;Falls negative Zahl Zweierkomplement + Flag
        ldi     r16,1
        add     r6,r16
        sts     TCFLAG,r16
SHTRH4: lds     r8,SORH        ;2 Zahl der Mul. (SOHR) in r8:r9
        lds     r9,SORH+1
        lsl     r8              ;8*SORH
        rol     r9
        lsl     r8
        rol     r9
        lsl     r8
        rol     r9              ;(1000+8*SORH)
        ldi     r16,LOW(1000)
        add     r8,r16
        ldi     r16,HIGH(1000)
        adc     r9,r16
        rcall   MUL22
        mov     r0,r2           ;/1000
        mov     r1,r3
        mov     r2,r4
        mov     r3,r5
        ldi     r16,LOW(1000)
        mov     r4,r16
        ldi     r16,HIGH(1000)
        mov     r5,r16
        rcall   DIV42
        ldi     r16,100
        mov     r2,r16
        rcall   DIV21

        lds     r16,TCFLAG      ;negative Zahl ,dann abziehen (SHTRH6)
        cpi     r16,1
        breq    SHTRH6
        add     r24,r1          ;Addieren mit Korrektur
        cpi     r24,100
        brlo    SHTRH5
        subi    r24,100
        inc     r25
SHTRH5: add     r25,r0
        rjmp    SHTRH8

SHTRH6: sub     r24,r1          ;Subtrahieren mit Korrektur
        sbrs    r24,7
        rjmp    SHTRH7
        ldi     r16,100
        add     r24,r16
        dec     r25
SHTRH7: sub     r25,r0
        


SHTRH8: mov     r0,r25           ;Quotient nach ASCII
        mov     r1,Zero
        ldi     r16,10
        mov     r2,r16
        rcall   DIV21
        ldi     r16,0x30
        add     r0,r16
        sts     HUM1+0,r0
        sts     HUM2+0,r0
        add     r1,r16
        sts     HUM1+1,r1
        sts     HUM2+1,r1
        ldi     r16,','         ;Komma
        sts     HUM1+2,r16
        mov     r0,r24          ;Rest nach ASCII
        mov     r1,Zero
        ldi     r16,10
        mov     r2,r16
        rcall   DIV21
        ldi     r16,0x30
        add     r0,r16
        sts     HUM1+3,r0
        add     r1,r16
        sts     HUM1+4,r1
        ldi     r16,'%'         ;%
        sts     HUM1+5,r16
        sts     HUM2+2,r16
        clr     r16             ;Nullbyte
        sts     HUM1+6,r16
        sts     HUM2+3,r16


        pop     r16
        ret

;-------------------------------------------------------------------------------
;       SHT75 Temperatur (14 Bit) messen, Daten in r25:r24 (dauert ungef. 320ms)
;       ASCII: 9 Byte xxx,xx°C + Nullbyte in SRAM Tabelle TEMP1
;              5 Byte xx°C + Nullbyte in SRAM Tabelle TEMP2 (gerundet)
;-------------------------------------------------------------------------------
SHTRTE: push    r16
        clr     r24             ;
        clr     r25
        ;Pins initialisieren
        sbi     SHTDDR,SHTSCK   ;SCK = Ausgang
        sbi     SHTDDR,SHTDATA  ;DATA = Ausgang
        ;Transmission Start                DATA           SCK
        sbi     SHTPORT,SHTSCK  ;                        -------
        sbi     SHTPORT,SHTDATA ;        -------                |
        rcall   W1us            ;               |               |
        cbi     SHTPORT,SHTDATA ;        -------                |
        rcall   W1us            ;       |                       |
        cbi     SHTPORT,SHTSCK  ;       |                -------
        rcall   W1us            ;       |               |
        rcall   W1us            ;       |               |
        sbi     SHTPORT,SHTSCK  ;       |                -------
        rcall   W1us            ;       |                       |
        sbi     SHTPORT,SHTDATA ;        -------                |
        rcall   W1us            ;               |               |
        cbi     SHTPORT,SHTSCK  ;               |        -------
        rcall   W1us            ;               |       |
        rcall   W1us            ;               |       |
        cbi     SHTPORT,SHTDATA ;        -------        |
        rcall   W1us            ;       |               |
        rcall   W1us            ;       |               |
        rcall   W1us            ;       |               |
        ;Adress 0b000           ;
        sbi     SHTPORT,SHTSCK  ;       |                -------
        rcall   W1us            ;       |                       | A2
        rcall   W1us            ;       |                       |
        cbi     SHTPORT,SHTSCK  ;       |                -------
        rcall   W1us            ;       |               |
        rcall   W1us            ;       |               |
        sbi     SHTPORT,SHTSCK  ;       |                -------
        rcall   W1us            ;       |                       | A1
        rcall   W1us            ;       |                       |
        cbi     SHTPORT,SHTSCK  ;       |                -------
        rcall   W1us            ;       |               |
        rcall   W1us            ;       |               |
        sbi     SHTPORT,SHTSCK  ;       |                -------
        rcall   W1us            ;       |                       | A0
        rcall   W1us            ;       |                       |
        cbi     SHTPORT,SHTSCK  ;       |                -------
        rcall   W1us            ;       |               |
        rcall   W1us            ;       |               |
        ;Command 00101 Measure Humidity
        sbi     SHTPORT,SHTSCK  ;       |                -------
        rcall   W1us            ;       |                       | C4
        rcall   W1us            ;       |                       |
        cbi     SHTPORT,SHTSCK  ;       |                -------
        rcall   W1us            ;       |               |
        rcall   W1us            ;       |               |
        sbi     SHTPORT,SHTSCK  ;       |                -------
        rcall   W1us            ;       |                       | C3
        rcall   W1us            ;       |                       |
        cbi     SHTPORT,SHTSCK  ;       |                -------
        rcall   W1us            ;       |               |
        rcall   W1us            ;       |               |
        sbi     SHTPORT,SHTSCK  ;       |                -------
        rcall   W1us            ;       |                       | C2
        rcall   W1us            ;       |                       |
        cbi     SHTPORT,SHTSCK  ;       |                -------
        rcall   W1us            ;       |               |
        sbi     SHTPORT,SHTDATA ;        -------        |
        rcall   W1us            ;               |       |
        sbi     SHTPORT,SHTSCK  ;               |        -------
        rcall   W1us            ;               |               | C1
        rcall   W1us            ;               |               |
        cbi     SHTPORT,SHTSCK  ;               |        -------
        rcall   W1us            ;               |       |
        rcall   W1us            ;               |       |
        sbi     SHTPORT,SHTSCK  ;               |        -------
        rcall   W1us            ;               |               | C0
        rcall   W1us            ;               |               |
        cbi     SHTPORT,SHTSCK  ;               |        -------
        cbi     SHTDDR,SHTDATA  ;DATA = Eingang
        sbi     SHTPORT,SHTDATA ;Pull-Up
        rcall   W1us            ;               |       |
        rcall   W1us            ;               |       |
        sbi     SHTPORT,SHTSCK  ;               |        -------
SHTRT1: sbic    SHTPIN,SHTDATA  ;        -------                | ACK
        rjmp    SHTRT1          ;       | ACK                   |
        cbi     SHTPORT,SHTSCK  ;       |                -------
SHTRT2: sbis    SHTPIN,SHTDATA  ;        -------        |
        rjmp    SHTRT2          ;               |       |
        rcall   W10ms           ;               |       |
SHTRT3: sbic    SHTPIN,SHTDATA  ;        -------        |
        rjmp    SHTRT3          ;       |               |
        rcall   W1us            ;       |               |
        ;Temperature Data
        sbi     SHTPORT,SHTSCK  ;       |                -------
        rcall   W1us            ;       |                       | H15
        rcall   W1us            ;       |                       |
        sbic    SHTPIN,SHTDATA  ;        *******                |
        ori     r25,0x80        ;        *******                |
        cbi     SHTPORT,SHTSCK  ;        *******         -------
        rcall   W1us            ;        *******        |
        rcall   W1us            ;        *******        |
        sbi     SHTPORT,SHTSCK  ;        *******         -------
        rcall   W1us            ;        *******                | H14
        rcall   W1us            ;        *******                |
        sbic    SHTPIN,SHTDATA  ;        *******                |
        ori     r25,0x40        ;        *******                |
        cbi     SHTPORT,SHTSCK  ;        *******         -------
        rcall   W1us            ;        *******        |
        rcall   W1us            ;        *******        |
        sbi     SHTPORT,SHTSCK  ;        *******         -------
        rcall   W1us            ;        *******                | H13
        rcall   W1us            ;        *******                |
        sbic    SHTPIN,SHTDATA  ;        *******                |
        ori     r25,0x20        ;        *******                |
        cbi     SHTPORT,SHTSCK  ;        *******         -------
        rcall   W1us            ;        *******        |
        rcall   W1us            ;        *******        |
        sbi     SHTPORT,SHTSCK  ;        *******         -------
        rcall   W1us            ;        *******                | H12
        rcall   W1us            ;        *******                |
        sbic    SHTPIN,SHTDATA  ;        *******                |
        ori     r25,0x10        ;        *******                |
        cbi     SHTPORT,SHTSCK  ;        *******         -------
        rcall   W1us            ;        *******        |
        rcall   W1us            ;        *******        |
        sbi     SHTPORT,SHTSCK  ;        *******         -------
        rcall   W1us            ;        *******                | H11
        rcall   W1us            ;        *******                |
        sbic    SHTPIN,SHTDATA  ;        *******                |
        ori     r25,0x08        ;        *******                |
        cbi     SHTPORT,SHTSCK  ;        *******         -------
        rcall   W1us            ;        *******        |
        rcall   W1us            ;        *******        |
        sbi     SHTPORT,SHTSCK  ;        *******         -------
        rcall   W1us            ;        *******                | H10
        rcall   W1us            ;        *******                |
        sbic    SHTPIN,SHTDATA  ;        *******                |
        ori     r25,0x04        ;        *******                |
        cbi     SHTPORT,SHTSCK  ;        *******         -------
        rcall   W1us            ;        *******        |
        rcall   W1us            ;        *******        |
        sbi     SHTPORT,SHTSCK  ;        *******         -------
        rcall   W1us            ;        *******                | H9
        rcall   W1us            ;        *******                |
        sbic    SHTPIN,SHTDATA  ;        *******                |
        ori     r25,0x02        ;        *******                |
        cbi     SHTPORT,SHTSCK  ;        *******         -------
        rcall   W1us            ;        *******        |
        rcall   W1us            ;        *******        |
        sbi     SHTPORT,SHTSCK  ;        *******         -------
        rcall   W1us            ;        *******                | H8
        rcall   W1us            ;        *******                |
        sbic    SHTPIN,SHTDATA  ;        *******                |
        ori     r25,0x01        ;        *******                |
        cbi     SHTPORT,SHTSCK  ;        *******         -------
        rcall   W1us            ;        *******        |
        rcall   W1us            ;        *******        |
        rcall   W1us            ;        *******        |
        rcall   W1us            ;        *******        |
        rcall   W1us            ;        *******        |
        sbi     SHTDDR,SHTDATA  ;DATA = Ausgang
        cbi     SHTPORT,SHTDATA;        | ACK           |
        rcall   W1us            ;       |               |
        sbi     SHTPORT,SHTSCK  ;       |                -------
        rcall   W1us            ;       |                       | ACK
        rcall   W1us            ;       |                       |
        cbi     SHTPORT,SHTSCK  ;       |                -------
        rcall   W1us            ;       |               |
        rcall   W1us            ;       |               |
        cbi     SHTDDR,SHTDATA  ;DATA = Eingang
        sbi     SHTPORT,SHTDATA ;Pull-Up
        sbi     SHTPORT,SHTSCK  ;       |                -------
        rcall   W1us            ;       |                       | H7
        rcall   W1us            ;       |                       |
        sbic    SHTPIN,SHTDATA  ;        *******                |
        ori     r24,0x80        ;        *******                |
        cbi     SHTPORT,SHTSCK  ;        *******         -------
        rcall   W1us            ;        *******        |
        rcall   W1us            ;        *******        |
        sbi     SHTPORT,SHTSCK  ;        *******         -------
        rcall   W1us            ;        *******                | H6
        rcall   W1us            ;        *******                |
        sbic    SHTPIN,SHTDATA  ;        *******                |
        ori     r24,0x40        ;        *******                |
        cbi     SHTPORT,SHTSCK  ;        *******         -------
        rcall   W1us            ;        *******        |
        rcall   W1us            ;        *******        |
        sbi     SHTPORT,SHTSCK  ;        *******         -------
        rcall   W1us            ;        *******                | H5
        rcall   W1us            ;        *******                |
        sbic    SHTPIN,SHTDATA  ;        *******                |
        ori     r24,0x20        ;        *******                |
        cbi     SHTPORT,SHTSCK  ;        *******         -------
        rcall   W1us            ;        *******        |
        rcall   W1us            ;        *******        |
        sbi     SHTPORT,SHTSCK  ;        *******         -------
        rcall   W1us            ;        *******                | H4
        rcall   W1us            ;        *******                |
        sbic    SHTPIN,SHTDATA  ;        *******                |
        ori     r24,0x10        ;        *******                |
        cbi     SHTPORT,SHTSCK  ;        *******         -------
        rcall   W1us            ;        *******        |
        rcall   W1us            ;        *******        |
        sbi     SHTPORT,SHTSCK  ;        *******         -------
        rcall   W1us            ;        *******                | H3
        rcall   W1us            ;        *******                |
        sbic    SHTPIN,SHTDATA  ;        *******                |
        ori     r24,0x08        ;        *******                |
        cbi     SHTPORT,SHTSCK  ;        *******         -------
        rcall   W1us            ;        *******        |
        rcall   W1us            ;        *******        |
        sbi     SHTPORT,SHTSCK  ;        *******         -------
        rcall   W1us            ;        *******                | H2
        rcall   W1us            ;        *******                |
        sbic    SHTPIN,SHTDATA  ;        *******                |
        ori     r24,0x04        ;        *******                |
        cbi     SHTPORT,SHTSCK  ;        *******         -------
        rcall   W1us            ;        *******        |
        rcall   W1us            ;        *******        |
        sbi     SHTPORT,SHTSCK  ;        *******         -------
        rcall   W1us            ;        *******                | H1
        rcall   W1us            ;        *******                |
        sbic    SHTPIN,SHTDATA  ;        *******                |
        ori     r24,0x02        ;        *******                |
        cbi     SHTPORT,SHTSCK  ;        *******         -------
        rcall   W1us            ;        *******        |
        rcall   W1us            ;        *******        |
        sbi     SHTPORT,SHTSCK  ;        *******         -------
        rcall   W1us            ;        *******                | H0
        rcall   W1us            ;        *******                |
        sbic    SHTPIN,SHTDATA  ;        *******                |
        ori     r24,0x01        ;        *******                |
        cbi     SHTPORT,SHTSCK  ;        *******         -------
        rcall   W1us            ;        *******        |
        rcall   W1us            ;        *******        |
        rcall   W1us            ;        *******        |
        rcall   W1us            ;        *******        |
        rcall   W1us            ;        *******        |
        sbi     SHTDDR,SHTDATA  ;DATA = Ausgang
        sbi     SHTPORT,SHTDATA;                |       |
        rcall   W1us            ;               |       |
        sbi     SHTPORT,SHTSCK  ;               |        -------
        rcall   W1us            ;               |               | ACK
        rcall   W1us            ;               |               |
        cbi     SHTPORT,SHTSCK  ;               |        -------
        rcall   W1us            ;               |       |
        ;Temperaturwert linearisieren und nach ASCII
        ;T = (SOT-4010)/100     4010 = 0x0FAA
        subi    r24,LOW(4010)
        sbci    r25,HIGH(4010)
        mov     r0,r24
        mov     r1,r25
        ldi     r16,100
        mov     r2,r16
        rcall   DIV21
        mov     r24,r1          ;Nachkommastellen (Rest) in r24
        mov     r25,r0          ;Quotient in r25
        sts     TEMP,r24        ;nach SRAM fuer Temperaturkompensation
        sts     TEMP+1,r25
        mov     r16,r0
        andi    r16,0x80        ;negative Temperatur?
        breq    SHTRT4
        ldi     r16,'-'
        sts     TEMP1+0,r16
        com     r25             ;Absolutwert durch 2er Komplement
        inc     r25
        rjmp    SHTRT5
SHTRT4: mov     r1,Zero         ;Positive Temperatur, Quotient nach ASCII
        ldi     r16,100
        mov     r2,r16
        rcall   DIV21
        mov     r25,r1
        ldi     r16,0x30
        add     r0,r16
        sts     TEMP1+0,r0
SHTRT5: mov     r0,r25
        mov     r1,Zero
        ldi     r16,10
        mov     r2,r16
        rcall   DIV21
        ldi     r16,0x30
        add     r0,r16
        sts     TEMP1+1,r0
        sts     TEMP2+0,r0
        add     r1,r16
        sts     TEMP1+2,r1
        sts     TEMP2+1,r1
        ldi     r16,','         ;Komma
        sts     TEMP1+3,r16
        mov     r0,r24
        mov     r1,Zero
        ldi     r16,10
        mov     r2,r16
        rcall   DIV21
        ldi     r16,0x30
        add     r0,r16
        sts     TEMP1+4,r0
        add     r1,r16
        sts     TEMP1+5,r1
        ldi     r16,0xDF        ;Gradzeichen LCD
        sts     TEMP1+6,r16
        sts     TEMP2+2,r16
        ldi     r16,'C'         ;C
        sts     TEMP1+7,r16
        sts     TEMP2+3,r16
        ldi     r16,' '         ;Leerzeichen
        sts     TEMP1+8,r16
        sts     TEMP2+4,r16
        clr     r16             ;Nullbyte
        sts     TEMP1+9,r16
        sts     TEMP2+5,r16

        pop     r16
        ret
